@@ -33,6 +33,11 @@ int main()
 #include <gtest/gtest.h>
 #include <gmock/gmock.h>
 
+#if defined(__llvm__)
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wgnu-zero-variadic-macro-arguments" /* for gtest under clang (yes, with '#pragma GCC') */
+#endif
+
 typedef unsigned short int rsbpp_half_idx_t;
 
 namespace rsb_internal
