@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2008-2021 Michele Martone
+Copyright (C) 2008-2022 Michele Martone
 
 This file is part of librsb.
 
@@ -33,6 +33,9 @@ If not, see <http://www.gnu.org/licenses/>.
 #include "rsb_common.h"
 #if RSB_WANT_XDR_SUPPORT
 //#ifndef RSB_HAVE_RPC_XDR_H
+#if RSB_HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
 #include <rpc/xdr.h>
 
 #pragma GCC visibility push(hidden)
