@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2008-2021 Michele Martone
+Copyright (C) 2008-2022 Michele Martone
 
 This file is part of librsb.
 
@@ -61,14 +61,15 @@ extern "C" {
 #ifdef RSB_HAVE_UNISTD_H
 #include <unistd.h>	/* getopt, gethostname (some system don't have it here!) */
 #endif /* RSB_HAVE_UNISTD_H */
-#ifdef RSB_HAVE_GETOPT_H 
-/* RSB_HAVE_GETOPT_LONG */
+#ifdef RSB_HAVE_GETOPT_LONG
+#ifdef RSB_HAVE_GETOPT_H
 #include <getopt.h>	/* getopt_long is not always available (e.g.: on AIX, or any non GNU system) */
-           typedef struct option rsb_option;
+#endif /* RSB_HAVE_GETOPT_H */
+           typedef struct option rsb_option_t;
            struct rsb_option_struct {
            	struct option ro;
            };
-#else /* RSB_HAVE_GETOPT_H  */
+#else /* RSB_HAVE_GETOPT_LONG */
 /*#undef required_argument*/
 /*#undef no_argument*/
 /*#undef optional_argument*/
@@ -78,18 +79,21 @@ extern "C" {
        extern char *optarg;
        extern int optind, opterr, optopt;
 
-           struct rsb_option_struct {
+           struct rsb_option {
                const char *name;
                int         has_arg;
                int        *flag;
                int         val;
            };
+           typedef struct rsb_option rsb_option_t;
+           struct rsb_option_struct {
+               struct rsb_option ro;
+           };
 /*!
  * \ingroup gr_internals
  * \brief An internal, helper structure.
  */
-typedef struct rsb_option_struct rsb_option;
-#endif /* RSB_HAVE_GETOPT_H */
+#endif /* RSB_HAVE_GETOPT_LONG */
 #include <ctype.h>	/*isdigit*/
 
 /*
