@@ -332,7 +332,7 @@ int rsb__dump_postscript(const int argc, char * const argv[])
 	*/
 	rsb_marf_t rflags = RSB_MARF_NOFLAGS;
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
-	rsb_option options[] = {
+	rsb_option_t options[] = {
 	    {"matrix-filename",	required_argument, NULL, 0x66},/* f */  
 	    {"dump-recursion",	no_argument, NULL, 'd'},/* NEW */
 	    {"width",required_argument	, NULL, 0x5757},/* W */
@@ -745,8 +745,10 @@ static rsb_err_t rsb__dump_postscript_operands(FILE*fd, const struct rsb_mtx_t*m
 %% colors for order lines:\n\
 /CLDOL { 0.1 0.1 1.0 setrgbcolor } bind def %%       C Layout Dashed Order Line\n\
 /CLDOLL{ 0.7 0.7 1.0 setrgbcolor } bind def %%       C Layout Dashed Order Line Light\n\
-/FLDOL { 0.0 0.6 0.0 setrgbcolor } bind def %% Fortran Layout Dashed Order Line\n\
-/FLDOLL{ 0.6 0.8 0.6 setrgbcolor } bind def %% Fortran Layout Dashed Order Line\n\
+/FLDOL { CLDOL  } bind def %% Fortran Layout Dashed Order Line\n\
+/FLDOLL{ CLDOLL } bind def %% Fortran Layout Dashed Order Line\n\
+%%/FLDOL { 0.0 0.6 0.0 setrgbcolor } bind def %% Fortran Layout Dashed Order Line\n\
+%%/FLDOLL{ 0.6 0.8 0.6 setrgbcolor } bind def %% Fortran Layout Dashed Order Line\n\
 ";
 	const char * mstr = "\
 \n\
@@ -1224,7 +1226,7 @@ rsb_err_t rsb__dump_postscript_recursion_from_mtx_t(FILE*fd, const char * filena
 	const rsb_rf_t ys = ((rsb_rf_t)height)/nrA, xs = ((rsb_rf_t)width)/ncA;
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 
-	if(fd && filename)
+	if( (fd && filename) || ( !fd && !filename) )
 	{
 		errval = RSB_ERR_BADARGS;
 		RSB_PERR_GOTO(err,RSB_ERRM_ES);
@@ -1694,7 +1696,14 @@ static rsb_err_t rsb_dump_postscript_from_mtx_t(FILE*fd, const struct rsb_mtx_t*
 	struct rsb_coo_mtx_t coo;
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 
+	if(!fd)
+	{
+		errval = RSB_ERR_BADARGS;
+		RSB_PERR_GOTO(err,RSB_ERRM_ES);
+	}
+
 	RSB_INIT_COO_FROM_MTX(&coo,mtxAp);
+
 	if(rsb__allocate_coo_matrix_t(&coo)!=&coo)
        	{
 	       	RSB_PERR_GOTO(err,RSB_ERRM_ES); 
@@ -1740,7 +1749,14 @@ rsb_err_t rsb__do_mtx_render(const char * filename, const struct rsb_mtx_t*mtxAp
 		       		fd = RSB_DEFAULT_FD;
 			}
 			else
+			{
 		       		fd = rsb__util_fopen(filename,"w");
+				if (!fd)
+				{
+					errval = RSB_ERR_BADARGS;
+					RSB_PERR_GOTO(err,RSB_ERRM_ES);
+				}
+			}
 
 			if( rflags == RSB_MARF_EPS || rflags == RSB_MARF_EPS_S || rflags == RSB_MARF_EPS_L )
 				RSB_DO_ERROR_CUMULATE(errval,rsb_dump_postscript_from_mtx_t(fd,mtxAp,1,1,pmWidth,pmHeight,1));
