@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2008-2021 Michele Martone
+Copyright (C) 2008-2022 Michele Martone
 
 This file is part of librsb.
 
@@ -376,7 +376,7 @@ int rsb__test_dump_main(const int argc,rsb_char_t *const argv[])
 	 * 
 	 * FIXME : this file functionality is already in test_matops.m4.
 	 **/
-	rsb_option options[] = {
+	rsb_option_t options[] = {
 	    {"block-rowsize",	required_argument, NULL, 'r'},  
 	    {"block-columns",	required_argument, NULL, 'c'},  
 	    {"type",		required_argument, NULL, 'T'},  
