@@ -95,6 +95,7 @@ TEST(Test_Init,init_basic)
 }
 
 
+#if RSB_HAVE_SETENV
 TEST(Test_Init,init_check_rsb_num_threads)
 {
 	auto dthf = [] (void) {
@@ -127,6 +128,51 @@ TEST(Test_Init,init_check_rsb_num_threads)
 #endif
 	}
 }
+#endif /* RSB_HAVE_SETENV */
+
+TEST(Test_Init,test__rsb_strerror_on_no_error)
+{
+	const auto buflen {256};
+	char buf[buflen];
+	const char c = '!';
+	buf[0] = c;
+	buf[1] = RSB_NUL;
+	const rsb_err_t errval = rsb_strerror_r(RSB_ERR_NO_ERROR, buf, buflen);
+
+	EXPECT_THAT( errval, Eq(RSB_ERR_NO_ERROR) );
+	EXPECT_THAT( buf[0], Eq(c) );
+	EXPECT_THAT( strlen(buf), Eq(1) );
+}
+
+TEST(Test_Init,test__rsb_strerror_on_errors)
+{
+	const auto buflen {256};
+	char buf[buflen];
+	const char c = '!';
+	buf[0] = c;
+	const rsb_err_t errvals = RSB_ERR_GENERIC_ERROR|RSB_ERR_BADARGS;
+	const rsb_err_t errval = rsb_strerror_r(errvals, buf, buflen);
+
+	EXPECT_THAT( errval, Eq(RSB_ERR_NO_ERROR) );
+	EXPECT_THAT( errvals, RSB_ERR_GENERIC_ERROR );
+	EXPECT_THAT( buf[0], Ne(c) );
+	EXPECT_THAT( strlen(buf), Gt(10) );
+	EXPECT_THAT( strlen(buf), Lt(256) );
+}
+
+TEST(Test_Init,test__rsb_strerror_on_error)
+{
+	const auto buflen {256};
+	char buf[buflen];
+	const char c = '!';
+	buf[0] = c;
+	const rsb_err_t errval = rsb_strerror_r(RSB_ERR_GENERIC_ERROR, buf, buflen);
+
+	EXPECT_THAT( errval, Eq(RSB_ERR_NO_ERROR) );
+	EXPECT_THAT( buf[0], Ne(c) );
+	EXPECT_THAT( strlen(buf), Gt(10) );
+	EXPECT_THAT( strlen(buf), Lt(256) );
+}
 
 
 class Test_After_Init: public ::testing::Test {
@@ -341,8 +387,10 @@ std::vector<NT> dense_va(const IT nr, const IT nc) {
 
 static int rsbg_clearenv(void) {
 #if RSB_ALLOW_INTERNAL_GETENVS
+#if RSB_HAVE_SETENV
 	unsetenv("RSB_SPLIT_SF");
 	setenv("RSB_WANT_COO2RSB_THREADS","4",1);
+#endif /* RSB_HAVE_SETENV */
 #endif /* RSB_ALLOW_INTERNAL_GETENVS */
 	return 0;
 }
@@ -486,6 +534,7 @@ TEST_F(testSquareSplit,split)
 	EXPECT_THAT( smn1, Ge( floor( smn0 * mfts ) + floor ( smn0 * mfts ) * 4 ) );
 }
 
+#if RSB_HAVE_SETENV
 #if RSB_ALLOW_INTERNAL_GETENVS
 TEST_F(testSquareSplit,split_two)
 {
@@ -541,6 +590,7 @@ TEST_F(testSquareSplit,split_default_on_wrong_env)
 	EXPECT_THAT( mtxAp_->all_leaf_matrices_n, Eq( smn0 + ( 4 - 1 ) * ( smn0 / 2 ) ) );
 }
 #endif /* RSB_ALLOW_INTERNAL_GETENVS */
+#endif /* RSB_HAVE_SETENV */
 
 TEST_F(testSquareSplit,split_manp)
 {
@@ -1341,6 +1391,16 @@ TEST_F(testCoo,SortedRowMajorBadType)
 	EXPECT_THAT( rsb__util_is_sorted_coo_as_row_major(coo_.IA, coo_.JA, coo_.nnz, RSB_NUMERICAL_TYPE_INVALID_TYPE, NULL, RSB_FLAG_WANT_ROW_MAJOR_ORDER), Eq(RSB_ERR_UNSUPPORTED_TYPE) );
 }
 
+TEST_F(testCoo,coo_sort)
+{
+	EXPECT_THAT( rsb_coo_sort(coo_.VA, coo_.IA, coo_.JA, coo_.nr, coo_.nc, coo_.nnz, coo_.typecode, RSB_FLAG_NOFLAGS ), Eq(RSB_ERR_NO_ERROR) );
+}
+
+TEST_F(testCoo,coo_sort_zero)
+{
+	EXPECT_THAT( rsb_coo_sort(coo_.VA, coo_.IA, coo_.JA, coo_.nnz, coo_.nr*0, coo_.nc*0, coo_.typecode, RSB_FLAG_NOFLAGS ), Eq(RSB_ERR_NO_ERROR) );
+}
+
 
 class test__do_util_sortcoo: public ::testing::Test {
 	protected:
@@ -2383,6 +2443,29 @@ TEST_F(test__vec_misc, are_similar)
 }
 #endif
 
+#ifdef RSB_NUMERICAL_TYPE_FLOAT_COMPLEX
+TEST_F(test__vec_misc, are_similar_parametric)
+{
+	using NT = typename std::complex<float>;
+	const std::vector<NT> a { 1.00000, 1.00000, 1.00000 };
+	const std::vector<NT> b { 1.00001, 1.00001, 1.00000 };
+	const std::vector<NT> c { 1.00010, 1.00010, 1.00000 };
+	rsb_err_t errval {};
+
+	errval = rsb__do_are_similar_parametric(a.data(), a.data(), 1, RSB_NUMERICAL_TYPE_FLOAT_COMPLEX, 1, 1, 0);
+	EXPECT_THAT( errval, Eq(RSB_ERR_NO_ERROR) );
+
+	errval = rsb__do_are_similar_parametric(a.data(), b.data(), 1, RSB_NUMERICAL_TYPE_FLOAT_COMPLEX, 1, 1, 0);
+	EXPECT_THAT( errval, Eq(RSB_ERR_NO_ERROR) );
+
+	errval = rsb__do_are_similar_parametric(a.data(), c.data(), 1, RSB_NUMERICAL_TYPE_FLOAT_COMPLEX, 1, 1, 0);
+	EXPECT_THAT( errval, Ne(RSB_ERR_NO_ERROR) );
+
+	errval = rsb__do_are_similar_parametric(a.data(), c.data(), 1, RSB_NUMERICAL_TYPE_FLOAT_COMPLEX, 1, 1, 1);
+	EXPECT_THAT( errval, Eq(RSB_ERR_NO_ERROR) );
+}
+#endif
+
 
 class test__err: public ::testing::Test {
 	protected:
