@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2008-2021 Michele Martone
+Copyright (C) 2008-2022 Michele Martone
 
 This file is part of librsb.
 
@@ -487,6 +487,30 @@ rsb_err_t rsb__do_init_inner(void)
 #ifdef RSB_WANT_CACHE_TIMER_GRANULARITY
 	rsb_global_session_handle.timer_granularity = rsb__timer_granularity(); /* for RSB_CACHED_TIMER_GRANULARITY */
 #endif /* RSB_WANT_CACHE_TIMER_GRANULARITY */
+#if RSB_WANT_OMP_RECURSIVE_KERNELS
+	if ( RSB_WANT_OMP_RECURSIVE_KERNELS != 0 )
+	{
+#ifndef _OPENMP
+#error There is something wrong with your configuration.. are you really compiling using OpenMP flags?
+#endif
+	}
+#endif /* RSB_WANT_OMP_RECURSIVE_KERNELS */
+#if RSB_USE_ASSERT
+#if RSB_WANT_OMP_RECURSIVE_KERNELS
+	{ /* --enable-debug */
+		int counter = 0;
+		#pragma omp parallel
+		{
+			counter ++;
+		}
+		if ( counter == 0 )
+		{
+			errval = RSB_ERR_INTERNAL_ERROR;
+			RSB_PERR_GOTO(err,"Severe OpenMP problem: #pragma omp parallel sections are being ignored!?\n");
+		}
+	}
+#endif /* RSB_WANT_OMP_RECURSIVE_KERNELS */
+#endif /* RSB_USE_ASSERT */
 #if RSB_WANT_OMP_RECURSIVE_KERNELS
 #if 0
 	#pragma omp parallel  RSB_NTC
