@@ -45,6 +45,8 @@ struct rsb_session_handle_t rsb_global_session_handle;
 
 #define RSB_INITIALIZE_CHECK_MTX_ERRP(ERRVALP)	if( ! rsb__do_was_initialized() ) { RSB_ERROR(RSB_ERRM_UL); RSB_INTERFACE_RETURN_MTX_ERRP(NULL,RSB_ERR_UNSUPPORTED_OPERATION,ERRVALP) }
 
+#define RSB_NULL_IF_NUL(P) (((P)&&!*(P))?NULL:(P)) /* convert '' to NULL */
+
 /*!
  * \internal
  * This library, currently, can be used by only one master thread.
@@ -706,7 +708,7 @@ rsb_err_t rsb_coo_sort(void *VA, rsb_coo_idx_t * IA, rsb_coo_idx_t * JA, rsb_nnz
 
 	   \param \rsb_wr_va_ia_ja_desc_msg
 	   \param \rsb_nnzA_inp_param_msg
-	   \param \rsb_nrcows_A_sparse_inp_param_msg
+	   \param \rsb_nrcows_A_sparse_inp_param_msg If any of \rsb_nrA or \rsb_ncA is zero, no sort occurs.
 	   \param \rsb_type_param_msg
 	   \param \rsb_flagsa_inp_param_msg If unsure, use #RSB_FLAG_NOFLAGS.
 	   \return \rsberrcodemsg
@@ -728,7 +730,7 @@ rsb_err_t rsb_coo_sort(void *VA, rsb_coo_idx_t * IA, rsb_coo_idx_t * JA, rsb_nnz
 	/* This is not the default and shall be rechecked. */
 	errval = rsb__util_sort_row_major_buffered(VA,IA,JA,nnzA,nrA,ncA,typecode,flags,NULL,0);
 #else
-	/* This is not the default, well tested. */
+	/* This is the default, well tested. */
 	errval = rsb__util_sort_row_major_inner(VA,IA,JA,nnzA,nrA,ncA,typecode,flagsA);
 #endif
 	RSB_INTERFACE_RETURN_ERR(errval)
@@ -838,6 +840,8 @@ rsb_err_t rsb_strerror_r(rsb_err_t errval, rsb_char_t * buf, size_t buflen)
 
 	   Writes a textual description of an error code in the specified string buffer.
 	   No more than buflen characters will be written (comprehensive of the terminating \c NUL character).
+	   No action will be performed on \c #RSB_ERR_NO_ERROR.
+	   Notice too that error flags cannot be added in the way flags are (e.g. \c (#RSB_ERR_GENERIC_ERROR|#RSB_ERR_BADARGS) evaluates to \c #RSB_ERR_GENERIC_ERROR).
 	  
 	   \param \rsb_errval_inp_param_msg
 	   \param \rsb_buf_inp_param_msg
@@ -962,7 +966,7 @@ rsb_err_t rsb_file_mtx_save(const struct rsb_mtx_t * mtxAp, const rsb_char_t * f
 	 */
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 	RSB_INTERFACE_PREAMBLE
-	errval = rsb__do_file_mtx_save(mtxAp,filename);
+	errval = rsb__do_file_mtx_save(mtxAp,RSB_NULL_IF_NUL(filename));
 	RSB_INTERFACE_RETURN_ERR(errval)
 }
 
@@ -1511,7 +1515,7 @@ rsb_err_t rsb_mtx_rndr(const char * filename, const struct rsb_mtx_t*mtxAp, rsb_
 	*/
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 	RSB_INTERFACE_PREAMBLE
-	errval = rsb__do_mtx_render(filename, mtxAp, pmWidth, pmHeight, rflags);
+	errval = rsb__do_mtx_render(RSB_NULL_IF_NUL(filename), mtxAp, pmWidth, pmHeight, rflags);
 	RSB_INTERFACE_RETURN_ERR(errval)
 }
 
