@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2008-2021 Michele Martone
+Copyright (C) 2008-2022 Michele Martone
 
 This file is part of librsb.
 
@@ -456,7 +456,7 @@ err:
 #if RSB_DO_WANT_PSORT_TIMING
 	dt = rsb_time();
 	tt+=dt;
-	RSB_INFO("pt:%lg  st:%lg  tt:%lg  mt:%lg ct:%lg\n",pt,st,tt,mt,ct);
+	RSB_INFO("tt:%lg  pt:%lg  st:%lg  mt:%lg ct:%lg\n",tt,pt,st,mt,ct);
 #endif /* RSB_DO_WANT_PSORT_TIMING */
 ret:
 	RSB_DO_ERR_RETURN(errval)
@@ -466,6 +466,9 @@ rsb_err_t rsb__util_sort_row_major_inner(void * RSB_RESTRICT VA, rsb_coo_idx_t *
 {
 	rsb_err_t errval = RSB_ERR_NO_ERROR;
 
+	if( nr == 0 || nc == 0 )
+		goto ret;
+
 #if RSB_WANT_SORT_PARALLEL_BUT_SLOW
 	if( rsb_global_session_handle.asm_sort_method > 0 && RSB_IS_ENOUGH_NNZ_FOR_PARALLEL_SORT(nnz) )
 		/* parallel and scaling but slow */
@@ -476,6 +479,7 @@ rsb_err_t rsb__util_sort_row_major_inner(void * RSB_RESTRICT VA, rsb_coo_idx_t *
 		/* not so parallel nor scaling but fast */
 		errval = rsb__util_sort_row_major_bucket_based_parallel(VA,IA,JA,nnz,nr,nc,typecode,flags);
 
+ret:
 	return errval;
 }
 /* @endcond */
