@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2008-2021 Michele Martone
+Copyright (C) 2008-2022 Michele Martone
 
 This file is part of librsb.
 
@@ -52,7 +52,7 @@ rsb_err_t rsb__do_switch_to_fullword_csr(struct rsb_mtx_t * mtxAp);
 
 /* #define RSB_COO_HALFWORDS_VALUES_PACK(LI,LJ)	((LJ)|((LI)<<RSB_COO_HALF_BITS_SIZE))*/ /* logical row and column index pack   (note: unused) */
 /* #define RSB_COO_HALFWORDS_VALUES_UNPACK_LI(LIJ)	((((LIJ))>>RSB_COO_HALF_BITS_SIZE)&~((-1)<<RSB_COO_HALF_BITS_SIZE))*/	/* logical row index unpack (note: unused) */
-#define RSB_COO_HALFWORDS_VALUES_UNPACK_UJ(LIJ)	(((LIJ) &~((-1)<<RSB_COO_HALF_BITS_SIZE)))	/* logical row index unpack (note: shall get rid of this) */
+#define RSB_COO_HALFWORDS_VALUES_UNPACK_UJ(LIJ)	(((LIJ) &~((~0U)<<RSB_COO_HALF_BITS_SIZE)))	/* logical row index unpack (note: shall get rid of this) */
 void rsb__do_switch_array_to_halfword_coo(rsb_coo_idx_t  *p, rsb_nnz_idx_t n, const rsb_half_idx_t off);
 void rsb__do_switch_array_to_fullword_coo(rsb_half_idx_t *p, rsb_nnz_idx_t n, const rsb_coo_idx_t off);
 
