@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2008-2021 Michele Martone
+Copyright (C) 2008-2022 Michele Martone
 
 This file is part of librsb.
 
@@ -104,15 +104,14 @@ If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>	/* memset */
 #endif
 
-#ifdef RSB_HAVE_SYS_RESOURCE_H 
-#define RSB_USE_RUSAGE 1
-#else
-#define RSB_USE_RUSAGE 0
-#endif
-
-#if RSB_USE_RUSAGE
+#if RSB_USE_GETRUSAGE
+#ifdef RSB_HAVE_SYS_RESOURCE_H
 #include <sys/resource.h>	/* getrusage */
 #endif
+#ifdef RSB_HAVE_SYS_TIME_H
+#include <sys/time.h>	/* getrusage */
+#endif
+#endif
 
 #ifdef RSB_HAVE_SYS_UTSNAME_H 
 #include <sys/utsname.h>	/* uname */
@@ -664,14 +663,12 @@ rsb_time_t rsb__timer_granularity(void)
 	/*!
 	   \ingroup gr_internals
 
-	 * Tries to estimate the granularity of the timing function (that is, its overhead) by measuring it.
-         * This value will be important when estimating minimal times for various self benchmarking operations.
-         * 
-	 * A test measuring the average and deviation of this parameter may be also useful.
-	 * It does not guarantee return with a broken timing function.
+	 * Estimate the granularity of the timing function (that is, its overhead) by measuring it.
+         * This value is important to set minimal benchmarking times.
+	 * No guarantee of return if the timing function is broken.
 	 * */
 	register double t = RSB_TIME_ZERO, t0 = RSB_TIME_ZERO;
-	int times = RSB_TIMER_GRANULARITY_TEST_TIMES;
+	const int times = RSB_TIMER_GRANULARITY_TEST_TIMES;
 	register int i = times;
 
 #if 0
@@ -1031,7 +1028,7 @@ rsb_int_t rsb__get_cache_levels_num(void)
 
 int rsb__getopt_long(
 	int argc, char * const argv[], const char *optstring,
-	const struct option *longopts, int *longindex)
+	const rsb_option_t *longopts, int *longindex)
 {
 	/*!
 	   \ingroup internals
@@ -1041,7 +1038,11 @@ int rsb__getopt_long(
 #ifdef RSB_HAVE_GETOPT_LONG
 	return getopt_long(argc,argv,optstring,longopts,longindex);
 #else /* RSB_HAVE_GETOPT_LONG */
+#ifdef RSB_HAVE_GETOPT
 	return getopt(argc,argv,optstring);	/* a remedy */
+#else /* RSB_HAVE_GETOPT */
+#error  Neither of getopt() nor getopt_long() detected!
+#endif /* RSB_HAVE_GETOPT */
 #endif /* RSB_HAVE_GETOPT_LONG */
 }
 
@@ -1323,7 +1324,7 @@ void rsb__print_trace(void)
 }
 #endif /* RSB_HAVE_EXECINFO_H */
 
-#if RSB_USE_RUSAGE
+#if RSB_USE_GETRUSAGE
 static rsb_time_t rsb_rstv(struct timeval*tvp)
 {
         register double t = 0.0;
@@ -1342,7 +1343,11 @@ rsb_err_t rsb__getrusage(void)
 
 	RSB_STDOUT("getrusage() stats:\n");
 	/*("ru_ixrss : %ld (integral shared memory size)\n",usage.ru_ixrss);*/
+#ifdef __APPLE__
+	/* ru_maxrss not necessarily available; unit might also differ... */
+#else
 	RSB_STDOUT("ru_maxrss: %ld (maximum resident set size -- MB)\n",usage.ru_maxrss / RSB_K);
+#endif
 	RSB_STDOUT("ru_stime : %0.4lgs (system CPU time used)\n",rsb_rstv(&usage.ru_stime));
 	RSB_STDOUT("ru_utime : %0.4lgs (user CPU time used)\n",rsb_rstv(&usage.ru_utime));
 #if 0
@@ -1352,12 +1357,12 @@ rsb_err_t rsb__getrusage(void)
 
 	return gru == 0 ? RSB_ERR_NO_ERROR : RSB_ERR_GENERIC_ERROR;
 }
-#else /* RSB_USE_RUSAGE */
+#else /* RSB_USE_GETRUSAGE */
 rsb_err_t rsb__getrusage(void)
 {
 	return RSB_ERR_NO_ERROR;
 }
-#endif /* RSB_USE_RUSAGE */
+#endif /* RSB_USE_GETRUSAGE */
 
 const rsb_char_t * rsb__getenv(const rsb_char_t * name)
 {
