@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2008-2021 Michele Martone
+Copyright (C) 2008-2022 Michele Martone
 
 This file is part of librsb.
 
@@ -104,6 +104,7 @@ static rsb_err_t aix_sys_info()
 
 
 
+#ifdef RSB_HAVE_SYSCONF
 static rsb_err_t get_sysconf_cacheinfo( long *cpa, long *cpb, long *cpc, int cac,  int cbc,  int ccc, int cl)
 {
 	/*!
@@ -122,6 +123,7 @@ static rsb_err_t get_sysconf_cacheinfo( long *cpa, long *cpb, long *cpc, int cac
 	}
 	return RSB_ERR_NO_ERROR;
 }
+#endif /* RSB_HAVE_SYSCONF */
 
 static long rsb_max_threads(void)
 {
@@ -232,7 +234,7 @@ rsb_err_t rsb__sys_info()
         if( mem_pages && pagesize)RSB_INFO("sysconf() : %zu bytes (%zu MB) of physical memory\n",tot_mem,(tot_mem)/(1024*1024));
 #if   defined(_SC_AVPHYS_PAGES)
         RSB_INFO("sysconf() : %zu available (free) physical pages\n",(size_t)sysconf(_SC_AVPHYS_PAGES));
-        RSB_INFO("sysconf() : %zu available (free) physical memory\n",(size_t)sysconf(_SC_AVPHYS_PAGES)*pagesize);
+        RSB_INFO("sysconf() : %zu available (free) physical memory\n",(size_t)(sysconf(_SC_AVPHYS_PAGES)*pagesize));
 #endif /* _SC_AVPHYS_PAGES */
 #endif /* RSB_HAVE_SYSCONF */
 	}
@@ -364,6 +366,11 @@ rsb_err_t rsb__sys_info()
 #else /* RSB_CFLAGS */
 	RSB_INFO("no CFLAGS info\n");
 #endif /* RSB_CFLAGS */
+#if   defined(RSB_CXXFLAGS)
+	RSB_INFO("CXXFLAGS : %s\n",RSB_CXXFLAGS);
+#else /* RSB_CXXFLAGS */
+	RSB_INFO("no CXXFLAGS info\n");
+#endif /* RSB_CXXFLAGS */
 #if   defined(RSB_CC)
 	RSB_INFO("CC       : %s\n",RSB_CC);
 #else /* RSB_CC */
