@@ -1,6 +1,6 @@
 /*
 
-Copyright (C) 2008-2021 Michele Martone
+Copyright (C) 2008-2022 Michele Martone
 
 This file is part of librsb.
 
@@ -428,6 +428,12 @@ rsb_err_t rsb__print_configuration_string_rsbench(const char *pn, rsb_char_t * c
 	sprintf(cs+strlen(cs),"MKL support: off.\n");
 #endif /* RSB_WANT_MKL */
 
+#if RSB_WANT_OMP_RECURSIVE_KERNELS
+	sprintf(cs+strlen(cs),"OpenMP support: on.\n");
+#else /* RSB_WANT_OMP_RECURSIVE_KERNELS */
+	sprintf(cs+strlen(cs),"OpenMP support: off.\n");
+#endif /* RSB_WANT_OMP_RECURSIVE_KERNELS */
+
 #if RSB_WANT_ARMPL
 	sprintf(cs+strlen(cs),"ARMPL support: on.\n");
 #else /* RSB_WANT_ARMPL */
@@ -464,6 +470,7 @@ err:
 	return errval;
 }
 
+#if RSB_HAVE_SETENV
 void rsb__setenv(const rsb_char_t * var_val)
 {
 	rsb_char_t * esp = NULL;
@@ -479,6 +486,7 @@ void rsb__setenv(const rsb_char_t * var_val)
 	if( ava )
 		free(ava);
 }
+#endif /* RSB_HAVE_SETENV */
 
 static int rsb__main_help(const int argc, char * const argv[], int default_program_operation, const char * program_codes, struct option *options)
 {
@@ -2208,7 +2216,7 @@ int rsb_mtx_ls_main(const int argc, char * argv[], rsb_bool_t want_latex);
 
 int main(const int argc, char * argv[])
 {
-	struct option options[] = {
+	rsb_option_t options[] = {
 	    {"help",			no_argument, NULL, 'h' },
 	    {"bench",			no_argument, NULL, 0x626e6368 }, /* bnch */
 	    {"matrix-operation",	required_argument, NULL, 'o' },
@@ -2308,8 +2316,13 @@ int main(const int argc, char * argv[])
 				return RSB_ERR_TO_PROGRAM_ERROR(rsb_mtx_ls_main(argc,argv,RSB_BOOL_TRUE));
 			break;
 			case 0x7365:
-			rsb__setenv(optarg);
-			aoc+=2;
+#if RSB_HAVE_SETENV
+				rsb__setenv(optarg);
+				aoc+=2;
+#else /* RSB_HAVE_SETENV */
+				RSB_STDERR("No setenv() built in -- terminating.\n");
+				return RSB_ERR_TO_PROGRAM_ERROR(RSB_ERR_GENERIC_ERROR);
+#endif /* RSB_HAVE_SETENV */
 			break;
 			case 0x7270720a:
 			case 0x72707268:
