@@ -26,8 +26,10 @@ If not, see <http://www.gnu.org/licenses/>.
 #include <algorithm>
 #include <rsb.hpp>
 #include <numeric>
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif /* HAVE_CONFIG_H */
 
-#define HAVE_GETOPT_H 1
 #ifdef HAVE_GETOPT_H
 #include <getopt.h> /* we want GNU's getopt_long */
 #endif /* HAVE_GETOPT_H */
@@ -427,6 +429,7 @@ RSBEP_IDXT MakeDataContiguous ()
 		return RSBEP_ERR_OK; // FIXME
 	}
 
+#if RSBEP_NO_STUB
 	RSBEP_ERRT ExtractGlobalRowCopy (long long GlobalRow, RSBEP_IDXT Length, RSBEP_IDXT &NumEntries, RSBEP_NUMT *Values, long long *Indices) const
 	{
 		// TODO: 
@@ -440,6 +443,7 @@ RSBEP_IDXT MakeDataContiguous ()
 		RSBEP_VI()
 		return RSBEP_ERR_GENERIC_ERROR;
 	}
+#endif
 
 	RSBEP_ERRT ExtractMyRowCopy (RSBEP_IDXT MyRow, RSBEP_IDXT Length, RSBEP_IDXT &NumEntries, RSBEP_NUMT *Values, RSBEP_IDXT *Indices) const
 	{
@@ -521,6 +525,7 @@ err:
 		return RSBEP_ERR_OK;
 	}
 
+#if RSBEP_NO_STUB
      	RSBEP_ERRT MaxNumEntries(RSBEP_IDXT MyRow, RSBEP_IDXT & NumEntries) const
 	{
 		RSBEP_VI()
@@ -534,6 +539,7 @@ err:
 			NumEntries = rnz > NumEntries ? rnz : NumEntries;
 		return RSBEP_ERR_OK;
 	}
+#endif
 
      	RSBEP_ERRT LeftScale(const Epetra_Vector& x)
 	{
@@ -602,8 +608,10 @@ err:
 		return nmd;
 	}
 
+#if RSBEP_NO_STUB
      	RSBEP_ERRT SetUseTranspose(bool UseTranspose)
 		RSBEP_INCOMPLETE_BODY
+#endif
 
      	bool UseTranspose() const
 	{
@@ -646,6 +654,7 @@ public:
 		return _InsertGlobalValues (GlobalRow, NumEntries, Values, Indices);
 	}
 
+#if RSBEP_NO_STUB
 	virtual RSBEP_ERRT InsertGlobalValues (long long GlobalRow, RSBEP_IDXT NumEntries, const RSBEP_NUMT *Values, const long long *Indices)
 		RSBEP_INCOMPLETE_BODY
 	virtual RSBEP_ERRT InsertGlobalValues (RSBEP_IDXT GlobalRow, RSBEP_IDXT NumEntries, RSBEP_NUMT *Values, RSBEP_IDXT *Indices)
@@ -679,6 +688,7 @@ public:
 		RSBEP_ERRT errval = spmv<rsb_err_t>(&y[0], &x[0], TransA);
 		return errval;
 	}
+#endif
 
 #if RSBEP_NO_STUB
   	virtual RSBEP_ERRT Multiply1(bool TransA, const Epetra_MultiVector& X, Epetra_MultiVector& Y) const
@@ -1049,7 +1059,6 @@ RSBEP_OS << " " << Coo_A.normInf() << "\n";
 		RSBEP_IDXT nr(Rsb_A.rows());
 		RSBEP_IDXT nc(Rsb_A.cols());
 		RSBEP_IDXT tnr = ( transA == RSB_TRANSPOSITION_N ) ? nr : nc;
-		RSBEP_IDXT tnc = ( transA == RSB_TRANSPOSITION_N ) ? nc : nr;
 		Epetra_RsbMultiVector Rsb_Y(tnr,ival,nrhsi);
 		Epetra_RsbMultiVector Coo_Y(tnr,ival,nrhsi);
 		Epetra_RsbMultiVector Csr_Y(tnr,ival,nrhsi);
@@ -1086,7 +1095,7 @@ int Rsb_Matrix_test_multimatrix_ms_mnrhs( RSBEP_IDXT nrhs, const char * const tr
 	/* A test matrix generator. */
 	int perrval = EXIT_FAILURE;
 	RSBEP_IDXT dim, ldim = 1, udim = RSBEP_GETENVI("RSBP_DMAX", 20);
-	bool wv = RSBEP_GETENVB("RSBP_VERBOSE",false);
+	//bool wv = RSBEP_GETENVB("RSBP_VERBOSE",false);
 	rsb_trans_t transA = RSBEP_INVALID_TRANS;
 	const char * transp = RSBP_NULL;
 bool dnsm = false; // debug non square matrices // FIXME: temporary
@@ -1098,12 +1107,11 @@ bool dnsm = false; // debug non square matrices // FIXME: temporary
 	for (dim = ldim; dim < udim; ++dim )
 	{
 		// pure diagonal
-		RSBEP_IDXT nnzT = dim;
 		std::vector<RSBEP_NUMT> VA(dim  ,0.0);
 		std::vector<RSBEP_IDXT> RP(dim+1,0);
 		std::vector<RSBEP_IDXT> JA(dim  ,0);
 		RSBEP_IDXT nzi = 0;
-		RSBEP_IDXT ri = 0, ci = 0;
+		RSBEP_IDXT ri = 0;
 
 		RP[ri]=0;
 		for ( ri = 0; ri < dim ; ++ri )
@@ -1126,7 +1134,6 @@ bool dnsm = false; // debug non square matrices // FIXME: temporary
 	{
 		// pure vertical
 		// please note that CSR constructor here uses maximal column index to extrapolate width
-		RSBEP_IDXT nnzT = dim;
 		std::vector<RSBEP_NUMT> VA(dim  ,0.0);
 		std::vector<RSBEP_IDXT> RP(dim+1,0);
 		std::vector<RSBEP_IDXT> JA(dim  ,0);
@@ -1153,7 +1160,6 @@ bool dnsm = false; // debug non square matrices // FIXME: temporary
 	for (RSBEP_IDXT cri = 0; cri < dim; ++cri ) // constant row index
 	{
 		// pure horizontal
-		RSBEP_IDXT nnzT = dim;
 		std::vector<RSBEP_NUMT> VA(dim  ,0.0);
 		std::vector<RSBEP_IDXT> RP(dim+1,0);
 		std::vector<RSBEP_IDXT> JA(dim  ,0);
@@ -1366,16 +1372,17 @@ ret:
 	return perrval;
 } /* Rsb_Matrix_demo */
 
+#if defined(RSBEP_NO_STUB) && defined(EPETRA_MPI)
 int Epetra_RsbMatrix_demo(int argc, char *argv[])
+#else
+int Epetra_RsbMatrix_demo()
+#endif
 {
 	std::ostream & os = RSBEP_OS;
-	int errval = RSB_ERR_NO_ERROR;
 	int perrval = EXIT_FAILURE;
 	const RSBEP_IDXT n = RSBEP_GETENVI("RSBP_N", 3);
 	const RSBEP_IDXT nrhs = RSBEP_GETENVI("RSBP_NRHS", 2);
 	const bool tune = RSBEP_GETENVI("RSBP_TUNE", false);
-	const RSBEP_IDXT nnz = ((n+1)*n)/2;
-   	RSBEP_ERRT ierr = RSBEP_ERR_OK;
 	RSBEP_IDXT MyRow = n - 1;
 	RSBEP_IDXT NumEntries = n;
 	RSBEP_NUMT xiv = 1.1, yiv = 99.;
@@ -1613,6 +1620,7 @@ int Epetra_RsbMatrix_demo(int argc, char *argv[])
 
 #if RSBEP_NO_STUB
 {
+	const RSBEP_IDXT nnz = ((n+1)*n)/2;
 	RSBEP_IDXT Length = n;
 	std::vector<RSBEP_NUMT> VA(nnz,0.0);
 	std::vector<RSBEP_IDXT> JA(nnz,0);
@@ -1688,7 +1696,7 @@ int Epetra_RsbMatrix_demo(int argc, char *argv[])
 #ifdef HAVE_GETOPT_H
 struct rsbp_option { struct option o;const char*dv;const char*od;};
 
-void rsbp_help(int argc, char * argv[], const struct option*options, const struct rsbp_option*ioptions, int nopts)
+void rsbp_help(char * argv[], const struct option*options, const struct rsbp_option*ioptions, int nopts)
 {
 	int c;
 	RSBP_PRINTF("Usage: {mpi-launcher} %s OPTIONS\n",argv[0]);
@@ -1719,6 +1727,7 @@ int main(int argc, char *argv[])
 	bool wanterctest = false;
 	bool wantrfbtest = false;
 
+	const rsb_char_t * filename = RSBEP_GETENVS("RSBP_FILENAME",RSBP_NULL);
 #ifdef HAVE_GETOPT_H
 	const struct rsbp_option ioptions[] = {
     		{"epetra",no_argument,RSBP_NULL,'e',RSBP_NULL,RSBP_NULL},
@@ -1728,13 +1737,12 @@ int main(int argc, char *argv[])
 	};
 	const int ion=sizeof(ioptions)/sizeof(struct rsbp_option);
 	struct option options[ion];
-	const rsb_char_t * filename = RSBEP_GETENVS("RSBP_FILENAME",RSBP_NULL);
 
 	for(int ioi=0;ioi<ion;++ioi)options[ioi]=ioptions[ioi].o;/* translation */
 
 	for (;;)
 	{
-		int c,opt_index,i;
+		int c,opt_index;
 
 		c = getopt_long(argc, argv, "efhr", options, &opt_index);
 		if (c == -1)
@@ -1752,7 +1760,7 @@ int main(int argc, char *argv[])
 			break;
 			case 'h':
 			default:
-				rsbp_help(argc,argv,options,ioptions,sizeof(options)/sizeof(options[0]));
+				rsbp_help(argv,options,ioptions,sizeof(options)/sizeof(options[0]));
 				goto ret;
 		}
 	}
@@ -1773,7 +1781,11 @@ int main(int argc, char *argv[])
 	if( wanterctest )
 	{
 		rsb_lib.meminfo();
+#if defined(RSBEP_NO_STUB) && defined(EPETRA_MPI)
 		perrval |= Epetra_RsbMatrix_demo(argc,argv);
+#else
+		perrval |= Epetra_RsbMatrix_demo();
+#endif
 		if( perrval != EXIT_SUCCESS)
 			RSBEP_PRERR_GOTO(pret,"")
 	}
@@ -1799,7 +1811,9 @@ pret:
 		if( perrval == EXIT_SUCCESS )
 			RSBEP_OS << "OK: terminating with no allocations registered in librsb\n";
 	}
+#ifdef HAVE_GETOPT_H
 ret:
+#endif /* HAVE_GETOPT_H */
 	return perrval;
 } /* main */
 
