@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2017-2021 Michele Martone
+Copyright (C) 2017-2022 Michele Martone
 
 This file is part of librsb.
 
@@ -41,7 +41,9 @@ If not, see <http://www.gnu.org/licenses/>.
  *  \author Michele Martone
  * */
 /*!
+ \if STANDALONEDOC
  \mainpage
+ \endif
 
   Header-only (\c rsb.hpp) wrapper classes for librsb (http://librsb.sf.net).
 */
@@ -135,7 +137,7 @@ If not, see <http://www.gnu.org/licenses/>.
 #define RSBP_CWMFHO() RSBP_LWCP(); RSBP_CFMH(1) /*     constant wrapper member function heading opening */
 #define RSBP_CDMFHO() RSBP_LWCP(); RSBP_CFMH(1) /*     constant derived member function heading opening */
 #define RSBP_CNMFHO()              RSBP_CNMH(1) /*     constant no-err. member function heading opening */
-#define RSBP_KWMFHO() RSBP_LWCP(); RSBP_CFMH(0) /* construction wrapper member function heading opening */
+#define RSBP_KWMFHO() RSBP_LWCP(); RSBP_CNMH(0) /* construction wrapper member function heading opening */
 #define RSBP_LWMFHO() RSBP_LWCP();  		/* construction wrapper member function heading opening */ \
 	rsb_err_t errval = RSB_ERR_NO_ERROR; \
 	RSBP_VWARNM(""); if( this->state_ != Begin ) { errval = RSB_ERR_BADARGS; RSBP_ERRORM("Invalid matrix state (did you initialize it in the proper sequence ?) !\n"); RSBP_THROW(errval); RSBP_MFR(errval); }  /* status check */
@@ -328,10 +330,19 @@ public:
 	{
 		/*!
 			Interface to rsb_lib_get_opt.
+			\warning	Only RSB_IO_WANT_MEMORY_HIERARCHY_INFO_STRING is supported for now.
 		*/
 		rsb_err_t errval = RSB_ERR_NO_ERROR;
 		const char * sp {nullptr};
-		RSB_REINIT_SINGLE_VALUE_GET(RSB_IO_WANT_MEMORY_HIERARCHY_INFO_STRING,&sp,errval);
+		if(iof != RSB_IO_WANT_MEMORY_HIERARCHY_INFO_STRING)
+		{
+			errval = RSB_ERR_UNIMPLEMENTED_YET;
+			RSBP_PERROR(errval);
+		}
+		else
+		{
+			RSB_REINIT_SINGLE_VALUE_GET(RSB_IO_WANT_MEMORY_HIERARCHY_INFO_STRING,&sp,errval);
+		}
 		RSBP_PERROR(errval);
 		return rsb_string_t(sp?sp:"");
 	}
@@ -424,7 +435,8 @@ public:
 		*/
 		rsb_err_t errval = RSB_ERR_NO_ERROR;
 
-		RSB_REINIT_SINGLE_VALUE_SET(RSB_IO_WANT_VERBOSE_EXIT,&stdout,errval);
+		auto of { stdout }; /* can't assume is an lvalue and therefore usable in '&stdout' */
+		RSB_REINIT_SINGLE_VALUE_SET(RSB_IO_WANT_VERBOSE_EXIT,&of,errval);
 		errval |= rsb_lib_exit(RSB_NULL_EXIT_OPTIONS);
 		if( errval != RSB_ERR_NO_ERROR )
 			RSBP_CRERR();
@@ -554,7 +566,7 @@ private:
 		rsb_flags_t flags = RSB_FLAG_NOFLAGS;
 
 		if ( sym == IsGen )
-			;
+			RSB_DO_FLAG_ADD( flags, RSB_FLAG_NOFLAGS);
 		if ( sym == IsHer )
 			RSB_DO_FLAG_ADD( flags, RSB_FLAG_HERMITIAN);
 		if ( sym == IsSym )
@@ -871,7 +883,7 @@ private:
 public:
 
 #if RSBP_WANT_CPP20
-	RSBP_RVT Err_t spmv(rsb_trans_t transA, const NT alpha, const std::span<const NT> X, const NT beta, std::span<NT> Y) const
+	RSBP_RVT Err_t spmv(rsb_trans_t transA, const NT alpha, const std::span<const NT> X, const NT beta, const std::span<NT> Y) const
 	{
 		/*!
 		\rsblib_based_on_prei rsb_spmv() \rsblib_based_on_post
@@ -990,7 +1002,7 @@ public:
 #endif /* RSBP_WANT_IMPLICIT_ORDER */
 
 #if RSBP_WANT_CPP20
-	RSBP_RVT Err_t spmm(rsb_trans_t transA, const NT alpha, rsb_coo_idx_t nrhs, rsb_flags_t order, const std::span<const NT> x, const NT beta, std::span<NT> y) const
+	RSBP_RVT Err_t spmm(rsb_trans_t transA, const NT alpha, rsb_coo_idx_t nrhs, rsb_flags_t order, const std::span<const NT> x, const NT beta, const std::span<NT> y) const
 	{
 		/*!
 		\rsblib_based_on_pred rsb_spmm() \rsblib_based_on_post
@@ -1331,27 +1343,79 @@ public:
 		/*!
 		\rsblib_based_on_pred rsb_mtx_get_coo() \rsblib_based_on_post
 
+		\rl_no_transa
 		\rl_deprecate
 		*/
 		RSBP_CWMFHO();
 
+		if(transA != RSB_TRANSPOSITION_N)
+			RSBP_PERROR(RSB_ERR_UNIMPLEMENTED_YET);
 		RSBP_CFE(rsb_mtx_get_coo(this->mtxAp_, VA, IA, JA, flags));
 		RSBP_CWMFRE();
 	}
 
+#if RSBP_WANT_CPP20
+	RSBP_RVT Err_t get_coo(rsb_trans_t transA, const std::span<NT> VA, const std::span<rsb_coo_idx_t> IA, const std::span<rsb_coo_idx_t> JA, rsb_flags_t flags ) const
+	{
+		/*!
+		\rsblib_based_on_pred rsb_mtx_get_coo() \rsblib_based_on_post
+
+		\rl_no_transa
+		*/
+		RSBP_CWMFHO();
+
+		if(transA != RSB_TRANSPOSITION_N)
+			RSBP_PERROR(RSB_ERR_UNIMPLEMENTED_YET);
+		RSBP_CFE(rsb_mtx_get_coo(this->mtxAp_, VA.data(), IA.data(), JA.data(), flags));
+		RSBP_CWMFRE();
+	}
+#endif /* RSBP_WANT_CPP20 */
+
 	RSBP_RVT RSBP_DEPRECATED Err_t get_csr(rsb_trans_t transA, NT * VA, rsb_coo_idx_t * RP, rsb_coo_idx_t * JA, rsb_flags_t flags ) const
 	{
 		/*!
 		\rsblib_based_on_pred rsb_mtx_get_csr() \rsblib_based_on_post
 
+		\rl_no_transa
 		\rl_deprecate
 		*/
 		RSBP_CWMFHO();
 
-		RSBP_CFE(rsb_mtx_get_csr(RSB_NUMERICAL_TYPE_SAME_TYPE, this->mtxAp_, VA, RP, JA, flags));
+		if(transA != RSB_TRANSPOSITION_N)
+		{
+			errval = RSB_ERR_UNIMPLEMENTED_YET;
+			RSBP_PERROR(errval);
+		}
+		else
+		{
+			RSBP_CFE(rsb_mtx_get_csr(RSB_NUMERICAL_TYPE_SAME_TYPE, this->mtxAp_, VA, RP, JA, flags));
+		}
 		RSBP_CWMFRE();
 	}
 
+#if RSBP_WANT_CPP20
+	RSBP_RVT RSBP_DEPRECATED Err_t get_csr(rsb_trans_t transA, const std::span<NT> VA, const std::span<rsb_coo_idx_t> RP, const std::span<rsb_coo_idx_t> JA, rsb_flags_t flags ) const
+	{
+		/*!
+		\rsblib_based_on_pred rsb_mtx_get_csr() \rsblib_based_on_post
+
+		\rl_no_transa
+		*/
+		RSBP_CWMFHO();
+
+		if(transA != RSB_TRANSPOSITION_N)
+		{
+			errval = RSB_ERR_UNIMPLEMENTED_YET;
+			RSBP_PERROR(errval);
+		}
+		else
+		{
+			RSBP_CFE(rsb_mtx_get_csr(RSB_NUMERICAL_TYPE_SAME_TYPE, this->mtxAp_, VA.data(), RP.data(), JA.data(), flags));
+		}
+		RSBP_CWMFRE();
+	}
+#endif /* RSBP_WANT_CPP20 */
+
 	RSBP_RVT RSBP_DEPRECATED Err_t get_rows_sparse(rsb_trans_t transA, const NT * alphap, NT * VA, rsb_coo_idx_t * IA, rsb_coo_idx_t * JA, rsb_coo_idx_t frA, rsb_coo_idx_t lrA, rsb_nnz_idx_t *rnzp, rsb_flags_t flags ) const
 	{
 		/*!
@@ -1534,7 +1598,7 @@ public:
 	}
 
 #if RSBP_WANT_CPP20
-	RSBP_RVT RSBP_DEPRECATED Err_t tune_spmm(rsb_real_t & sf, rsb_trans_t transA, const NT alpha, rsb_coo_idx_t nrhs, rsb_flags_t order, const std::span<const NT> B, rsb_nnz_idx_t ldB, const NT beta, std::span<NT> C, rsb_nnz_idx_t ldC)
+	RSBP_RVT RSBP_DEPRECATED Err_t tune_spmm(rsb_real_t & sf, rsb_trans_t transA, const NT alpha, rsb_coo_idx_t nrhs, rsb_flags_t order, const std::span<const NT> B, rsb_nnz_idx_t ldB, const NT beta, const std::span<NT> C, rsb_nnz_idx_t ldC)
 	{
 		/*!
 		\rsblib_based_on_prei rsb_tune_spmm() \rsblib_based_on_post
@@ -1549,7 +1613,7 @@ public:
 		RSBP_CWMFRE();
 	}
 
-	RSBP_RVT Err_t tune_spmm(rsb_real_t & sf, rsb_trans_t transA, const NT alpha, rsb_coo_idx_t nrhs, rsb_flags_t order, const std::span<const NT> B, const NT beta, std::span<NT> C)
+	RSBP_RVT Err_t tune_spmm(rsb_real_t & sf, rsb_trans_t transA, const NT alpha, rsb_coo_idx_t nrhs, rsb_flags_t order, const std::span<const NT> B, const NT beta, const std::span<NT> C)
 	{
 		/*!
 		\rsblib_based_on_pred rsb_tune_spmm() \rsblib_based_on_post
@@ -1567,7 +1631,7 @@ public:
 		RSBP_MDR();
 	}
 
-	RSBP_RVT RSBP_DEPRECATED Err_t tune_spmm(/*RsbMatrix * mtxp,*/ rsb_real_t *sfp, rsb_int_t *tnp, rsb_int_t maxr, rsb_time_t maxt, rsb_trans_t transA, const NT alpha, rsb_coo_idx_t nrhs, rsb_flags_t order, const std::span<const NT> B, rsb_nnz_idx_t ldB, const NT beta, std::span<NT> C, rsb_nnz_idx_t ldC)
+	RSBP_RVT RSBP_DEPRECATED Err_t tune_spmm(/*RsbMatrix * mtxp,*/ rsb_real_t *sfp, rsb_int_t *tnp, rsb_int_t maxr, rsb_time_t maxt, rsb_trans_t transA, const NT alpha, rsb_coo_idx_t nrhs, rsb_flags_t order, const std::span<const NT> B, rsb_nnz_idx_t ldB, const NT beta, const std::span<NT> C, rsb_nnz_idx_t ldC)
 	{
 		/*!
 		\rsblib_based_on_prei rsb_tune_spmm() \rsblib_based_on_post
